# -----------------------------------------------------------------------------
#
# Classes that contains the settings available for CAEN FELib Scope.
#
# Author    : Stephen Bolanos
# Date      : 2023/08/24
#
# Notes:
#   - Refer to CAEN FELib Scope Parameters User Manual for further explanation
# -----------------------------------------------------------------------------

class CAENScopeParams:
    START_SOURCE = [
        "EncodedClkIn", # Start from CLK-IN/SYNC connector on the front panel. 
                        # This is a 4-pin connector (LVDS signals) used to propagate 
                        # the reference clock (typ. 62.5 MHz) and a Sync 
                        # signal. The rising edge of the Sync starts the acquisition, 
                        # that lasts until the Sync returns low (falling edge).
        "SINlevel",     # Start from SIN (1=run, 0=stop)
        "SINedge",      # Start from SIN (rising edge = run; stop from SW)
        "SWcmd",        # Start from SW
        "LVDS",         # Start from LVDS
        "FirstTrigger", # Start on first trigger (stop from SW)
        "P0"            # Start from P0 (backplane)
    ]
    ACQ_TRIG_SOURCE = [
        "TrgIn",            # Front Panel TRGIN
        "P0",               # Trigger from P0 (backplane)
        "SWTrg",            # Software Trigger
        "LVDS",             # LVDS trgin
        "ITLA",             # Internal Trigger Logic A: combination of channel self-triggers
        "ITLB",             # Internal Trigger Logic B: combination of channel self-triggers
        "ITLA_AND_ITLB",    # Second level Trigger logic making the AND of ITL A and B
        "ITLA_OR_ITLB",     # Second level Trigger logic making the OR of ITL A and B
        "EncodedClk",       # Not implemented (encoded CLK-IN trigger)
        "GPIO",             # Front Panel GPIO
        "TestPulse"         # Internal Test Pulse
        ]
    TRIG_ID_MODE = [
        "TriggerCnt", 
        "EventCnt", 
        "LVDSpattern"
        ]
    CLOCK_SOURCE = [
        "Internal",         # Local oscillator, 62.5 MHz
        "FPClkIn",          # Front Panel Clock input
        "P0ClkIn",          # Not implemented (clock from P0 VME backplane)
        "Link",             # Not implemented (clock recovery from Eth or Opt Link)
        "DIPswitchSel"      # Not implemented (clock source decided by dip switches on board)
        ]     
    WAVE_DATA_SOURCE = [
        "ADC_DATA",         # Data from the ADC (normal operating mode)
        "ADC_TEST_TOGGLE",  # Toggle between 0x5555 and 0xAAAA (test mode)
        "ADC_TEST_RAMP",    # 16-bit ramp pattern (test mode)
        "ADC_TEST_SIN",     # 8-point sine wave test pattern
        "IPE",              # Not implemented
        "Ramp",             # Data from a ramp generator (see user manual for more info)
        "SquareWave",       # Internally generated programmable square wave
        "ADC_TEST_PRBS"     # 16-bit PRBS generated by a 23-bit PRBS pattern generator (test mode)
    ]
    DEC_FACTOR = [
        1,      # Decimation disabled (default)
        2,      # Sampling Frequency / 2
        4,      # Sampling Frequency / 4
        8,      # Sampling Frequency / 8
        16,     # Sampling Frequency / 16
        32,     # Sampling Frequency / 32
        64,     # Sampling Frequency / 64
        128,    # Sampling Frequency / 128
        256,    # Sampling Frequency / 256
        512,    # Sampling Frequency / 512
        1024    # Sampling Frequency / 1024
    ]
    ACTIVE_ENDPOINT = [
        "raw",
        "scope"
    ]